<?php
/**
 * Discount Feed WooCommerce
 *
 * @package    Discount Feed WooCommerce
 * @subpackage DiscountFeedWooCommerce
/*
	Copyright (c) 2018- Katsushi Kawamori (email : dodesyoswift312@gmail.com)
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; version 2 of the License.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

$discountfeedwoocommerce = new DiscountFeedWooCommerce();

/** ==================================================
 * Main Functions
 */
class DiscountFeedWooCommerce {

	/** ==================================================
	 * Construct
	 *
	 * @since 1.00
	 */
	public function __construct() {

		add_action( 'init', array( $this, 'add_custom_feed' ) );
		add_action( 'wp_head', array( $this, 'discount_feed_woo_alternate' ) );
		add_filter( 'gutenberg_can_edit_post_type', array( $this, 'gutenberg_post_type' ), 10, 2 );

		add_action( 'DiscountFeedWooCommerceHook', array( $this, 'discount_auto_post' ) );
		register_activation_hook( plugin_dir_path( __DIR__ ) . 'discountfeedwoocommerce.php', array( $this, 'cron_start' ) );
		register_deactivation_hook( plugin_dir_path( __DIR__ ) . 'discountfeedwoocommerce.php', array( $this, 'cron_stop' ) );

	}

	/** ==================================================
	 * Add feed
	 *
	 * @since 1.00
	 */
	public function add_custom_feed() {

		add_feed( 'dcfeedwoo', array( $this, 'load_create_feed' ) );

	}

	/** ==================================================
	 * Load Database Custom Post
	 *
	 * @return object $cposts
	 * @since 1.00
	 */
	private function db_custom_post_load() {

		global $wpdb;
		$cposts = $wpdb->get_results(
			"
							SELECT	ID, post_date, post_content, post_title, post_status
							FROM	$wpdb->posts
							WHERE	post_type LIKE '%%dcfeedwoo%%'
							AND		post_status = 'publish'
							ORDER BY post_date DESC
							"
		);

		return $cposts;

	}

	/** ==================================================
	 * Create Custom Post Feed Items
	 *
	 * @since 1.00
	 */
	public function load_create_feed() {

		$feed_title = __( 'Discount Information', 'discount-feed-woocommerce' );

		$duration = 'hourly';
		$update_period = apply_filters( 'rss_update_period', $duration );

		$frequency = '1';
		$update_frequency = apply_filters( 'rss_update_frequency', $frequency );

		$cposts = $this->db_custom_post_load();
		if ( function_exists( 'wp_date' ) ) {
			$diff_time  = wp_date( 'O' );
		} else {
			$diff_time  = date_i18n( 'O' );
		}

		header( 'Content-Type: ' . feed_content_type( 'rss2' ) . '; charset=' . get_option( 'blog_charset' ), true );

		?><?php echo '<?'; ?>xml version="1.0" encoding="<?php echo esc_attr( get_option( 'blog_charset' ) ); ?>"<?php echo '?>'; ?>

<rss version="2.0"
	xmlns:content="http://purl.org/rss/1.0/modules/content/"
	xmlns:wfw="http://wellformedweb.org/CommentAPI/"
	xmlns:dc="http://purl.org/dc/elements/1.1/"
	xmlns:atom="http://www.w3.org/2005/Atom"
	xmlns:sy="http://purl.org/rss/1.0/modules/syndication/"
	xmlns:slash="http://purl.org/rss/1.0/modules/slash/"
	>
	<channel>
	<title><?php echo esc_html( $feed_title . ' &#187; ' . get_bloginfo( 'name' ) ); ?></title>
	<atom:link href="<?php echo esc_url( get_feed_link( 'dcfeedwoo' ) ); ?>" rel="self" type="application/rss+xml" />
	<link><?php echo esc_url( get_feed_link( 'dcfeedwoo' ) ); ?></link>
	<description><?php echo esc_html( $feed_title ); ?></description>
	<lastBuildDate><?php echo esc_html( mysql2date( 'D, d M Y H:i:s ', $cposts[0]->post_date, false ) . $diff_time ); ?></lastBuildDate>
	<language><?php echo esc_html( get_bloginfo( 'language' ) ); ?></language>
	<sy:updatePeriod><?php echo esc_html( $update_period ); ?></sy:updatePeriod>
	<sy:updateFrequency><?php echo esc_html( $update_frequency ); ?></sy:updateFrequency>
<?php
		foreach ( $cposts as $cpost ) {
			$date = $cpost->post_date;
?>
<item>
<title><?php echo esc_html( $cpost->post_title ); ?></title>
<link><?php echo esc_url( get_permalink( $cpost->ID ) ); ?></link>
<description><![CDATA[<?php echo wp_kses_post( $cpost->post_content ); ?>]]></description>
<guid><?php echo esc_url( get_permalink( $cpost->ID ) ); ?></guid>
<pubDate><?php echo esc_html( mysql2date( 'D, d M Y H:i:s ', $date, false ) . $diff_time ); ?></pubDate>
</item>
<?php
		}
?>
</channel>
</rss>
<?php

	}

	/** ==================================================
	 * Feed alternate
	 *
	 * @since 1.00
	 */
	public function discount_feed_woo_alternate() {

		if ( class_exists( 'WooCommerce' ) ) {
			echo '<link rel="alternate" type="' . esc_attr( feed_content_type() ) . '" title="' . esc_attr( __( 'Discount Information', 'discount-feed-woocommerce' ) . ' ' . __( 'Feed', 'discount-feed-woocommerce' ) . ' &raquo; ' . get_bloginfo( 'name' ) ) . '" href="' . esc_url( get_feed_link( 'dcfeedwoo' ) ) . "\" />\n";
		}

	}

	/** ==================================================
	 * Load Database Post Meta Sale
	 *
	 * @return object $sales
	 * @since 1.00
	 */
	private function db_postmeta_sale_load() {

		global $wpdb;
		$sales = $wpdb->get_results(
			"
						SELECT	post_id, meta_key, meta_value
						FROM	$wpdb->postmeta
						WHERE	meta_key LIKE '%%_sale_price%%'
						"
		);

		return $sales;

	}

	/** ==================================================
	 * Load Database Post Meta Coupon
	 *
	 * @return object $coupons
	 * @since 1.00
	 */
	private function db_postmeta_coupon_load() {

		global $wpdb;
		$coupons = $wpdb->get_results(
			"
						SELECT	post_id, meta_key, meta_value
						FROM	$wpdb->postmeta
						WHERE	meta_key = 'coupon_amount'
						"
		);

		return $coupons;

	}

	/** ==================================================
	 * Discount Auto Post
	 *
	 * @since 1.00
	 */
	public function discount_auto_post() {

		if ( class_exists( 'WooCommerce' ) ) {

			$discountfeedwoocommerce_option = get_option( 'discount_feed_woocommerce' );
			$currency = get_option( 'woocommerce_currency' );
			$currency_symbol = get_woocommerce_currency_symbol( $currency );

			$sale_html = '<div><b>' . __( 'Sale', 'discount-feed-woocommerce' ) . '</b></div>';
			$woo_sale_text_tag = $discountfeedwoocommerce_option['sale_text_tag'];
			$sales = $this->db_postmeta_sale_load();
			foreach ( $sales as $sale ) {
				if ( '_sale_price' === $sale->meta_key ) {
					if ( ! empty( $sale->meta_value ) ) {
						$sale_html .= $this->post_woo_sale_item( $sale, $currency_symbol, $woo_sale_text_tag );
					}
				}
			}
			$sale_html .= '<br />';

			$coupon_html = '<div><b>' . __( 'Coupon', 'discount-feed-woocommerce' ) . '</b></div>';
			$woo_coupon_text_tag = $discountfeedwoocommerce_option['coupon_text_tag'];
			$coupons = $this->db_postmeta_coupon_load();
			foreach ( $coupons as $coupon ) {
				if ( 'coupon_amount' === $coupon->meta_key ) {
					if ( ! empty( $coupon->meta_value ) ) {
						$coupon_html .= $this->post_woo_coupon_item( $coupon, $currency_symbol, $woo_coupon_text_tag );
					}
				}
			}

			$dc_info = __( 'Discount Information', 'discount-feed-woocommerce' );
			if ( function_exists( 'wp_date' ) ) {
				$date = wp_date( 'Y-m-d H:i:s' );
			} else {
				$date = date_i18n( 'Y-m-d H:i:s' );
			}
			$title = $date . ' ' . $dc_info;

			$discount_post = array(
				'post_content'  => $sale_html . $coupon_html,
				'post_name'     => $date . 'dcfeedwoo',
				'post_title'    => $title,
				'post_status'   => 'publish',
				'post_type'     => 'dcfeedwoo_product',
				'post_author'   => 1,
			);
			wp_insert_post( $discount_post );

		}

	}

	/** ==================================================
	 * Gutenberg post type
	 *
	 * @param bool   $can_edit  can_edit.
	 * @param string $post_type  post_type.
	 * @since 1.03
	 */
	public function gutenberg_post_type( $can_edit, $post_type ) {
		if ( 'dcfeedwoo_product' == $post_type ) {
			return true;
		}
		return false;
	}

	/** ==================================================
	 * Sale item
	 *
	 * @param object $sale  sale.
	 * @param string $currency_symbol  currency_symbol.
	 * @param string $woo_sale_text_tag  woo_sale_text_tag.
	 * @return string $post_html
	 * @since 1.00
	 */
	private function post_woo_sale_item( $sale, $currency_symbol, $woo_sale_text_tag ) {

		$woo_sale_datas = array();

		$sale_product = get_the_title( $sale->post_id );
		$sale_link = get_permalink( $sale->post_id );
		$product_thumb_id = get_post_meta( $sale->post_id, '_thumbnail_id', true );
		$sale_img_tag = null;
		if ( ! empty( $product_thumb_id ) ) {
			$sale_img_url = wp_get_attachment_image_src( $product_thumb_id, 'thumbnail', false );
			if ( $sale_img_url ) {
				$sale_img_tag = '<img style="float: left; margin-right:5px;" width="20" height="' . intval( 20 * $sale_img_url[2] / $sale_img_url[1] ) . '" src ="' . $sale_img_url[0] . '"/>';
			}
		}
		$sale_html = '<a href="' . $sale_link . '">' . $sale_img_tag . $sale_product . '</a>';

		$regular_price = get_post_meta( $sale->post_id, '_regular_price', true );
		$sale_price = get_post_meta( $sale->post_id, '_sale_price', true );
		$to_date_ux = get_post_meta( $sale->post_id, '_sale_price_dates_to', true );

		$woo_sale_datas['regular_price'] = sprintf( get_woocommerce_price_format(), $currency_symbol, $regular_price );
		$woo_sale_datas['sale_price'] = sprintf( get_woocommerce_price_format(), $currency_symbol, $sale_price );

		$dis_amount = sprintf( get_woocommerce_price_format(), $currency_symbol, intval( $regular_price - $sale_price ) );
		$dis_rate = intval( ( ( $regular_price - $sale_price ) / $regular_price ) * 100 );
		/* translators: Discount amount */
		$woo_sale_datas['dis_amount'] = sprintf( __( '%1$s %2$s', 'discount-feed-woocommerce' ), $dis_amount, __( 'OFF', 'countdown-woocommerce-sale' ) );
		/* translators: Discount rate */
		$woo_sale_datas['dis_rate'] = sprintf( __( '%1$d&#37; %2$s', 'discount-feed-woocommerce' ), $dis_rate, __( 'OFF', 'countdown-woocommerce-sale' ) );

		if ( ! empty( $to_date_ux ) ) {
			$current_date_ux = time();
			if ( function_exists( 'wp_date' ) ) {
				$woo_sale_datas['to_date'] = wp_date( 'Y-m-d H:i:s', $to_date_ux, new DateTimeZone( 'UTC' ) );
			} else {
				$woo_sale_datas['to_date'] = date_i18n( 'Y-m-d H:i:s', $to_date_ux, false );
			}
			$interval_day_ux = $to_date_ux - $current_date_ux;
			$woo_sale_datas['interval_day'] = intval( $interval_day_ux / ( 60 * 60 * 24 ) ) . __( 'Days', 'discount-feed-woocommerce' );
			$woo_sale_datas['interval_time'] = gmdate( 'H:i:s', $interval_day_ux % ( 60 * 60 * 24 ) );
		}

		$woo_sale_text = null;
		if ( $woo_sale_datas ) {
			$woo_sale_text = $woo_sale_text_tag;
			foreach ( $woo_sale_datas as $item => $woo_sale ) {
				$woo_sale_text = str_replace( '%' . $item . '%', $woo_sale, $woo_sale_text );
			}
			preg_match_all( '/%(.*?)%/', $woo_sale_text, $woo_sale_text_per_match );
			foreach ( $woo_sale_text_per_match as $key1 ) {
				foreach ( $key1 as $key2 ) {
					$woo_sale_text = str_replace( '%' . $key2 . '%', '', $woo_sale_text );
				}
			}
		}
		$post_html = '<div>' . $sale_html . ' : ' . $woo_sale_text . '</div>';

		return $post_html;

	}

	/** ==================================================
	 * Coupon item
	 *
	 * @param object $coupon  coupon.
	 * @param string $currency_symbol  currency_symbol.
	 * @param string $woo_coupon_text_tag  woo_coupon_text_tag.
	 * @return string $post_html
	 * @since 1.00
	 */
	private function post_woo_coupon_item( $coupon, $currency_symbol, $woo_coupon_text_tag ) {

		$woo_coupon_datas = array();

		$woo_coupon_datas['coupon_code'] = get_the_title( $coupon->post_id );
		$discount_type = get_post_meta( $coupon->post_id, 'discount_type', true );
		$coupon_amount = get_post_meta( $coupon->post_id, 'coupon_amount', true );
		if ( 'percent' === $discount_type ) {
			$coupon_amount = $coupon_amount . '&#37;';
		} else {
			$coupon_amount = sprintf( get_woocommerce_price_format(), $currency_symbol, $coupon_amount );
		}
		/* translators: Coupon amount */
		$woo_coupon_datas['discount'] = sprintf( __( '%1$s %2$s', 'discount-feed-woocommerce' ), $coupon_amount, __( 'OFF', 'countdown-woocommerce-sale' ) );

		$woo_coupon_datas['expiry_date'] = get_post_meta( $coupon->post_id, 'expiry_date', true );

		$woo_coupon_datas['product'] = null;
		$product_ids = get_post_meta( $coupon->post_id, 'product_ids', true );
		if ( ! empty( $product_ids ) ) {
			$product_ids = explode( ',', $product_ids );
			foreach ( $product_ids as $product_id ) {
				$coupon_product = get_the_title( $product_id );
				$coupon_link = get_permalink( $product_id );
				$product_thumb_id = get_post_meta( $product_id, '_thumbnail_id', true );
				$coupon_img_tag = null;
				if ( ! empty( $product_thumb_id ) ) {
					$coupon_img_url = wp_get_attachment_image_src( $product_thumb_id, 'thumbnail', false );
					if ( $coupon_img_url ) {
						$coupon_img_tag = '<img style="float: left; margin-right:5px;" width="20" height="' . intval( 20 * $coupon_img_url[2] / $coupon_img_url[1] ) . '" src ="' . $coupon_img_url[0] . '"/>';
					}
				}
				$coupon_html = '<a href="' . $coupon_link . '">' . $coupon_img_tag . $coupon_product . '</a>';
				$woo_coupon_datas['product'] .= $coupon_html . ' ';
			}
		}

		$woo_coupon_datas['exclude_product'] = null;
		$exclude_product_ids = get_post_meta( $coupon->post_id, 'exclude_product_ids', true );
		if ( ! empty( $exclude_product_ids ) ) {
			$exclude_product_ids = explode( ',', $exclude_product_ids );
			foreach ( $exclude_product_ids as $exclude_product_id ) {
				$exclude_coupon_product = get_the_title( $exclude_product_id );
				$woo_coupon_datas['exclude_product'] .= $exclude_coupon_product . ' ';
			}
		}

		$woo_coupon_datas['product_categories'] = null;
		$product_categories_ids = get_post_meta( $coupon->post_id, 'product_categories', true );
		if ( ! empty( $product_categories_ids ) ) {
			$cat_name = null;
			foreach ( $product_categories_ids as $product_categories_id ) {
				$term = get_term( $product_categories_id, 'product_cat' );
				$cat_name .= $term->name . ',';
			}
			$woo_coupon_datas['product_categories'] = rtrim( $cat_name, ',' );
		}

		$woo_coupon_datas['exclude_product_categories'] = null;
		$exclude_product_categories_ids = get_post_meta( $coupon->post_id, 'exclude_product_categories', true );
		if ( ! empty( $exclude_product_categories_ids ) ) {
			$edcat_name = null;
			foreach ( $exclude_product_categories_ids as $exclude_product_categories_id ) {
				$term = get_term( $exclude_product_categories_id, 'product_cat' );
				$edcat_name .= $term->name . ',';
			}
			$woo_coupon_datas['exclude_product_categories'] = rtrim( $edcat_name, ',' );
		}

		$woo_coupon_text = null;
		if ( $woo_coupon_datas ) {
			$woo_coupon_text = $woo_coupon_text_tag;
			foreach ( $woo_coupon_datas as $item => $woo_coupon ) {
				$woo_coupon_text = str_replace( '%' . $item . '%', $woo_coupon, $woo_coupon_text );
			}
			preg_match_all( '/%(.*?)%/', $woo_coupon_text, $woo_coupon_text_per_match );
			foreach ( $woo_coupon_text_per_match as $key1 ) {
				foreach ( $key1 as $key2 ) {
					$woo_coupon_text = str_replace( '%' . $key2 . '%', '', $woo_coupon_text );
				}
			}
		}
		$post_html = '<div>' . $woo_coupon_text . '</div>';

		return $post_html;

	}

	/** ==================================================
	 * Cron Start
	 *
	 * @return bool $result fail >> false | success >> void
	 * @since 1.00
	 */
	public function cron_start() {

		$discountfeedwoocommerce_option = get_option( 'discount_feed_woocommerce' );

		if ( ! wp_next_scheduled( 'DiscountFeedWooCommerceHook' ) ) {
			$result = wp_schedule_event( time(), $discountfeedwoocommerce_option['schedule'], 'DiscountFeedWooCommerceHook' );
		} else {
			wp_clear_scheduled_hook( 'DiscountFeedWooCommerceHook' );
			$result = wp_schedule_event( time(), $discountfeedwoocommerce_option['schedule'], 'DiscountFeedWooCommerceHook' );
		}

		return $result;

	}

	/** ==================================================
	 * Cron Stop
	 *
	 * @since 1.00
	 */
	public function cron_stop() {

		wp_clear_scheduled_hook( 'DiscountFeedWooCommerceHook' );

	}

}


