<?php
/**
 * Discount Feed WooCommerce
 *
 * @package    DiscountFeedWooCommerce
 * @subpackage DiscountFeedWooCommerce Widget
/*
	Copyright (c) 2018- Katsushi Kawamori (email : dodesyoswift312@gmail.com)
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; version 2 of the License.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

add_action(
	'widgets_init',
	function() {
		register_widget( 'DiscountFeedWooCommerceWidgetItem' );
	}
);

/** ==================================================
 * Widget
 *
 * @since 1.00
 */
class DiscountFeedWooCommerceWidgetItem extends WP_Widget {

	/** ==================================================
	 * Construct
	 *
	 * @since 1.00
	 */
	public function __construct() {

		parent::__construct(
			'DiscountFeedWooCommerceWidgetItem', /* Base ID */
			'Discount Feed WooCommerce', /* Name */
			array( 'description' => __( 'Entries of RSS feed from "Discount Feed WooCommerce".', 'discount-feed-woocommerce' ) ) /* Args */
		);

	}

	/** ==================================================
	 * Widget
	 *
	 * @param array $args args.
	 * @param array $instance instance.
	 * @since 1.00
	 */
	public function widget( $args, $instance ) {

		$before_widget = $args['before_widget'];
		$before_title  = $args['before_title'];
		$after_title   = $args['after_title'];
		$after_widget  = $args['after_widget'];

		$title = apply_filters( 'widget_title', $instance['title'] );

		if ( $title ) {

			echo wp_kses_post( $before_widget );
			echo wp_kses_post( $before_title . esc_html( $title ) . $after_title );

			$feedwidget_tbl = get_option( 'discount_feed_woocommerce_widget' );
			foreach ( $feedwidget_tbl as $key => $xmlurl ) {
				if ( isset( $instance[ $key ] ) && $instance[ $key ] ) {
					$checkbox[ $key ] = apply_filters( 'widget_checkbox', $instance[ $key ] );
				}
				if ( isset( $checkbox[ $key ] ) && $checkbox[ $key ] ) {
					$feedtitle = __( 'Discount Information', 'discount-feed-woocommerce' );
					?>
					<div class="textwidget"><ul><li>
					<a href="<?php echo esc_url( $xmlurl ); ?>" style="text-decoration: none; word-break: break-all;" title="<?php echo esc_attr( $feedtitle ); ?>"><span class="dashicons dashicons-rss"></span><?php echo esc_html( $feedtitle ); ?></a>
					</li></ul></div>
					<?php
				}
			}
			echo wp_kses_post( $after_widget );
		}

	}

	/** ==================================================
	 * Update
	 *
	 * @param array $new_instance new_instance.
	 * @param array $old_instance old_instance.
	 * @since 1.00
	 */
	public function update( $new_instance, $old_instance ) {

		$instance = $old_instance;
		$instance['title'] = wp_strip_all_tags( $new_instance['title'] );
		$feedwidget_tbl = get_option( 'discount_feed_woocommerce_widget' );
		foreach ( $feedwidget_tbl as $key => $xmlurl ) {
			$instance[ $key ] = wp_strip_all_tags( $new_instance[ $key ] );
		}
		return $instance;

	}

	/** ==================================================
	 * Form
	 *
	 * @param array $instance instance.
	 * @since 1.00
	 */
	public function form( $instance ) {

		if ( isset( $instance['title'] ) ) {
			$title = $instance['title'];
		} else {
			$title = null;
		}
		?>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"><?php esc_html_e( 'Title' ); ?>:</label>
			<input class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'title' ) ); ?>" type="text" value="<?php echo esc_attr( $title ); ?>" />
		</p>
		<table>
		<?php

		$feedwidget_tbl = get_option( 'discount_feed_woocommerce_widget' );
		foreach ( $feedwidget_tbl as $key => $xmlurl ) {
			if ( isset( $instance[ $key ] ) ) {
				$checkbox[ $key ] = esc_attr( $instance[ $key ] );
			} else {
				$checkbox[ $key ] = null;
			}
			$feedtitle = __( 'Discount Information', 'discount-feed-woocommerce' );
			?>
			<tr>
			<td align="left" valign="middle" nowrap>
				<label for="<?php echo esc_attr( $this->get_field_id( $key ) ); ?> ">
				<input class="widefat" id="<?php echo esc_attr( $this->get_field_id( $key ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( $key ) ); ?>" type="checkbox"<?php checked( $key, $checkbox[ $key ] ); ?> value="<?php echo esc_attr( $key ); ?>" />
				<?php echo esc_html( $feedtitle ); ?></label>
			</td>
			</tr>
			<?php
		}
		?>
		</table>
		<?php

	}

}


